﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using IsbnCheckSum.Resources;

namespace IsbnCheckSum.Calculation
{
	/// <summary>
	/// Calculates checksum values for valid ISBN codes
	/// </summary>
	public class NumericMod11CheckSumCalculator : ICalculateCheckSums
	{
		private string baseSequence;

		/// <summary>
		/// Calculates the checksum for a given ISBN code
		/// </summary>
		public char CalculateCheckSum(string baseSequence)
		{
			this.baseSequence = baseSequence;

			this.VerifyInputIsValid();

			var sum = this.CalculateTotal();
			var mod11 = (sum % 11).ToString();

			return mod11 != "10" ? mod11[0] : 'X';
		}

		private void VerifyInputIsValid()
		{
			if (baseSequence.IsEmpty())
			{
				throw new ArgumentException(NumericMod11Messages.MustHaveBaseSequence);
			}

			// Strip out any hyphens as we don't care about them in this code block
			this.baseSequence = this.baseSequence.Replace("-", String.Empty);

			Regex onlyNumeric = new Regex(@"^[0-9]+$");
			if (!onlyNumeric.IsMatch(baseSequence))
			{
				throw new ArgumentException(NumericMod11Messages.MustOnlyHaveNumbers);
			}
		}

		private int CalculateTotal()
		{
			int sum = 0;
			int iteration = 1;

			foreach (var value in baseSequence)
			{	
				sum += Int32.Parse(value.ToString()) * iteration++;
			}

			return sum;
		}
	}
}
