﻿using FluentAssertions;
using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using IsbnCheckSum.Resources;
using IsbnCheckSum.Calculation;

namespace IsbnCheckSum.Tests.Domain.Calculation
{
	public class NumericMod11CheckSumCalculatorTests
	{
		[TestFixture]
		public class CalculateCheckSumMethod : CheckSumCalulatorMethodBase
		{
			[Test]
			public void CorrectlyCalculatesCheckSum()
			{
				// Arrange
				string input = "312586734";
				char expected = '7';

				// Act
				char result = this.Calculator.CalculateCheckSum(input);

				// Assert
				result.Should().Be(expected);
			}

			[Test]
			public void ReturnsXWhenCheckSumIsGreaterThanNine()
			{
				// Arrange
				string input = "354899122";
				char expected = 'X';

				// Act
				char result = this.Calculator.CalculateCheckSum(input);

				// Assert
				result.Should().Be(expected);
			}

			[Test]
			public void IgnoresHyphensWhenCalculatingSum()
			{
				// Arrange
				string input = "3-548-991-22";
				char expected = 'X';

				// Act
				char result = this.Calculator.CalculateCheckSum(input);

				// Assert
				result.Should().Be(expected);
			}

			[Test]
			public void ThrowsExceptionForNonNumericCodes()
			{
				// Arrange
				string input = "123a54df";

				// Act + Assert
				this.ShouldThrow<ArgumentException>(input, NumericMod11Messages.MustOnlyHaveNumbers);
			}

			[Test]
			public void ThrowsExceptionForEmptyInput()
			{
				// Arrange
				string input = string.Empty;

				// Act + Assert
				this.ShouldThrow<ArgumentException>(input, NumericMod11Messages.MustHaveBaseSequence);
			}

			[Test]
			public void ThrowsExceptionForNullInput()
			{
				// Arrange
				string input = null;

				// Act + Assert
				this.ShouldThrow<ArgumentException>(input, NumericMod11Messages.MustHaveBaseSequence);
			}

			private void ShouldThrow<TException>(string input, string expectedMessage) where TException : Exception
			{
				// Act
				Action action = () => this.Calculator.CalculateCheckSum(input);

				// Assert
				action.ShouldThrow<TException>()
					.WithMessage(expectedMessage);
			}
		}

		public abstract class CheckSumCalulatorMethodBase
		{
			protected NumericMod11CheckSumCalculator Calculator { get; set; }

			[SetUp]
			protected virtual void SetUp()
			{
				this.Calculator = new NumericMod11CheckSumCalculator();
			}
		}
	}
}
