﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FluentAssertions;
using IsbnCheckSum.Calculation;
using IsbnCheckSum.Validation;
using Moq;
using NUnit.Framework;

namespace IsbnCheckSum.Tests.Domain
{
	public class IsbnCodeTests
	{
		[TestFixture]
		public class PublicConstructorMethod : IsbnCodeMethodBase
		{
			[Test]
			public void CalculatesCheckSumWhenOnlyBaseSequenceIsSubmitted()
			{
				// Arrange
				string input = "312586734";
				string expectedBaseSequence = input;

				this.Calculator
					.Setup(x => x.CalculateCheckSum(It.IsAny<string>()))
					.Returns('7');
				this.FormatValidator
					.Setup(x => x.CodeContainsCheckSum(It.IsAny<string>()))
					.Returns(false);

				// Act
				var codeObject = new IsbnCode(input, this.Calculator.Object, this.FormatValidator.Object);

				// Assert
				codeObject.BaseSequence.Should().Be(expectedBaseSequence);
				codeObject.CheckSumDigit.Should().NotBe(Char.MinValue);
				this.Calculator.Verify(
					x => x.CalculateCheckSum(It.IsAny<string>()), 
					Times.Once);
				this.FormatValidator.Verify(
					x => x.CodeContainsCheckSum(It.IsAny<string>()),
					Times.Once);
			}

			[Test]
			public void ParsesCheckSumFromCodeWhenFullCodeIsSubmitted()
			{
				// Arrange
				string input = "3125867347";
				string expectedBaseSequence = "312586734";
				char expectedCheckDigit = '7';

				this.FormatValidator
					.Setup(x => x.CodeContainsCheckSum(It.IsAny<string>()))
					.Returns(true);

				// Act
				var codeObject = new IsbnCode(input, this.Calculator.Object, this.FormatValidator.Object);

				// Assert
				codeObject.BaseSequence.Should().Be(expectedBaseSequence);
				codeObject.CheckSumDigit.Should().Be(expectedCheckDigit);
				this.Calculator.Verify(
					x => x.CalculateCheckSum(It.IsAny<string>()),
					Times.Never);
				this.FormatValidator.Verify(
					x => x.CodeContainsCheckSum(It.IsAny<string>()),
					Times.Once);
			}
		}

		// TODO: Tests to verify that validation utilises the FormatValidator, as well as verifies that check digit is correct

		public abstract class IsbnCodeMethodBase
		{
			protected Mock<ICalculateCheckSums> Calculator { get; set; }
			protected Mock<ICanValidateIsbnCodeFormats> FormatValidator { get; set; }

			[SetUp]
			protected virtual void SetUp() 
			{
				this.Calculator = new Mock<ICalculateCheckSums>();
				this.FormatValidator = new Mock<ICanValidateIsbnCodeFormats>();
			}
		}
	}
}
